using WaterLily
using StaticArrays
function hover(L=2^5;Re=250,U=1,amp=π/4,ϵ=0.5,thk=2ϵ+√2)
    # Line segment SDF
    function sdf(x,t)
        y = x .- SA[0,clamp(x[2],-L/2,L/2)]
        √sum(abs2,y)-thk/2
    end
    # Oscillating motion and rotation
    function map(x,t)
        α = amp*cos(t*U/L); R = SA[cos(α) sin(α); -sin(α) cos(α)]
        R * (x - SA[3L-L*sin(t*U/L),4L])
    end
    Simulation((6L,6L),(0,0),L;U,ν=U*L/Re,body=AutoBody(sdf,map),ϵ)
end

sim = hover();
sim_step!(sim,π)
include("TwoD_plots.jl")
a = sim.flow.σ;
@inside a[I] = WaterLily.curl(3,I,sim.flow.u)*sim.L/sim.U
flood(a[inside(a)],clims=(-5,5))
body_plot!(sim)